/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.container.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.tinygroup.container.BaseObject;
import org.tinygroup.container.Category;

@SuppressWarnings("rawtypes")
public class CategoryImpl<T extends Category> extends BaseObjectImpl implements
		Category<T> {

	private T parent;
	private List<T> subList;

	public String getType() {
		return this.getClass().getSimpleName();
	}

	public T getParent() {
		return parent;
	}

	public void setParent(T parent) {
		this.parent = parent;
	}

	public List<T> getSubList() {
		return this.subList;
	}

	public void setSubList(List<T> subList) {
		this.subList = subList;
	}

	public List<T> getSubList(Comparator<BaseObject> comparator) {
		List<T> newList = new ArrayList<T>(subList);
		Collections.sort(newList, comparator);
		return newList;

	}

	/**
	 * 递归查找，要么自己包含，要么子孙包含
	 */
	public boolean containsSub(T category) {
		if (containsDirectly(category)) {
			return true;
		}
		if (subList != null) {
			for (Category<T> cat : subList) {
				if (cat.containsSub(category)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 要么自己包含，要要父亲的长辈包含
	 */
	public boolean belong(T category) {
		return category.containsSub(this);
	}

	public boolean containsDirectly(T category) {
		if (subList == null) {
			return false;
		}
		return subList.contains(category);
	}

	public boolean belongDirectly(T category) {
		return category.containsDirectly(this);
	}

}
